/*
Battery Pack Quadruped Op Program
Written 01/08/2023 By Andres Torres
Updated 02/09/2023 by Andres Torres
*/

//////////////////////////////////////////////
//        RemoteXY include library          //
//////////////////////////////////////////////

// RemoteXY select connection mode and include library 
#define REMOTEXY_MODE__SOFTSERIAL
#include <SoftwareSerial.h>

#include <RemoteXY.h>

// RemoteXY connection settings 
#define REMOTEXY_SERIAL_RX 4
#define REMOTEXY_SERIAL_TX 5
#define REMOTEXY_SERIAL_SPEED 9600


// RemoteXY configurate  
#pragma pack(push, 1)
uint8_t RemoteXY_CONF[] =   // 284 bytes
  { 255,8,0,3,0,21,1,16,31,0,129,0,15,4,70,6,8,66,97,116,
  116,101,114,121,32,80,97,99,107,32,81,117,97,100,114,117,112,101,100,0,
  1,1,21,21,8,12,9,31,32,32,70,111,114,32,32,0,1,1,6,35,
  12,7,9,31,32,32,32,32,76,101,102,116,32,32,32,32,0,1,1,30,
  37,12,7,9,31,32,32,32,32,82,105,103,104,116,32,32,32,32,0,1,
  1,19,45,8,12,9,31,32,32,66,97,99,32,32,0,1,1,82,37,12,
  8,10,8,32,32,32,32,82,105,103,104,116,32,32,32,32,0,1,1,61,
  37,12,8,10,8,32,32,32,32,76,101,102,116,32,32,32,32,0,4,32,
  48,35,7,22,10,26,66,192,77,21,16,7,63,26,70,18,33,16,14,4,
  4,37,0,70,18,55,16,14,4,4,37,0,129,0,80,17,10,3,8,66,
  97,116,116,101,114,121,0,129,0,45,12,11,3,8,83,101,110,115,111,114,
  115,0,129,0,43,55,11,3,8,83,116,114,105,100,101,32,72,101,105,103,
  104,116,0,129,0,73,33,9,3,8,82,111,116,97,116,101,0,129,0,9,
  27,7,3,8,77,111,118,101,0,1,0,47,25,9,9,77,8,67,101,110,
  116,101,114,0 };
  
// this structure defines all the variables and events of your control interface 
struct {

    // input variables
  uint8_t move_forward; // =1 if button pressed, else =0 
  uint8_t move_left; // =1 if button pressed, else =0 
  uint8_t move_right; // =1 if button pressed, else =0 
  uint8_t move_backward; // =1 if button pressed, else =0 
  uint8_t rotate_right; // =1 if button pressed, else =0 
  uint8_t rotate_left; // =1 if button pressed, else =0 
  int8_t strd_hgt; // =-100..100 slider position 
  uint8_t strd_cnt; // =1 if button pressed, else =0 

    // output variables
  int8_t batt_lvl; // =0..100 level position 
  uint8_t ant_l; // led state 0 .. 1 
  uint8_t ant_r; // led state 0 .. 1 

    // other variable
  uint8_t connect_flag;  // =1 if wire connected, else =0 

} RemoteXY;
#pragma pack(pop)

/////////////////////////////////////////////
//           END RemoteXY include          //
/////////////////////////////////////////////

#include "HCPCA9685.h"

#define  I2CAdd 0x40

HCPCA9685 HCPCA9685(I2CAdd);

// servo connections
const int FRH = 0;  const int FRK = 1;
const int RRH = 2;  const int RRK = 3;
const int RLK = 12; const int RLH = 13;
const int FLK = 14; const int FLH = 15;

int i;                  // counter
int move_s = 150;       // movement speed - lower is faster (100 is the lowest)
int strd_height = 50;   // stride height default 50
long j_mag = 0;         // jitter magnitude
int ant_left = 1;       // antenna sensor input pin 2
int ant_right = 1;      // antenna sensor input pin 3
int BT_status = 0;      // bluetooth module connection status
float battRaw = 0;      // raw analog data
float battLevel = 5;    // calculated battery level
float battSamp[5];      // sampling for smoothing output
float battFull = 6;     // full tank 
float battEmpty = 4.6;  // low battery threshold
float battCal = .007;   // scale multiplier

void setup() 
{
  Serial.begin(9600);

  RemoteXY_Init (); 
  
  HCPCA9685.Init(SERVO_MODE);
  HCPCA9685.Sleep(false);

  pinMode(6, INPUT);    // Bluetooth state pin
  pinMode(2, INPUT);    // left antenna
  pinMode(3, INPUT);    // right antenna
  pinMode(A3, INPUT);   // battery in

  // position check space - comment moveStand at the end and uncomment each one by one to check movement stability
  //moveRotLeft();
  //moveRotRight();
  //moveForward();
  //moveBackward();
  //moveLeft();
  //moveRight();
}

void loop() 
{
  RemoteXY_Handler ();
  
  // stride height setting
  strd_height = map(RemoteXY.strd_hgt, -100, 100, -20, 100); // Serial.println(strd_height);
  if (RemoteXY.strd_cnt == 1)
    {
        RemoteXY.strd_hgt = 0;
    }
  
  // joystick inputs
    joystick:
    if (RemoteXY.rotate_left == 1) {RemoteXY.rotate_left = 0; moveRotLeft();}
    if (RemoteXY.rotate_right == 1) {RemoteXY.rotate_right = 0; moveRotRight();}
    if (RemoteXY.move_forward == 1) {RemoteXY.move_forward = 0; moveForward();}
    if (RemoteXY.move_backward == 1) {RemoteXY.move_backward = 0; moveBackward();}
    if (RemoteXY.move_left == 1) {RemoteXY.move_left = 0; moveLeft();}
    if (RemoteXY.move_right == 1) {RemoteXY.move_right = 0; moveRight();}

  // Bluetooth status check and LED indicator
  BT_status = digitalRead(6); digitalWrite(13, BT_status);
  
  // antennae sensor check
  ant_sen_check:
  ant_left = (!digitalRead(2)); RemoteXY.ant_l = ant_left;
  ant_right = (!digitalRead(3)); RemoteXY.ant_r = ant_right;

  // antenae sensor check reaction
  if (BT_status == 0){
    if (ant_left == 1) {moveBackward(); moveRotRight(); goto ant_sen_check;}
    if (ant_right == 1) {moveBackward(); moveRotLeft(); goto ant_sen_check;}
  }
  
  // battery Level check  
  if (i > 0) {i--;} else {i = 4;}
  battRaw = analogRead(A3); battSamp[0] = battRaw; battSamp[i] = battSamp[i - 1];       // smoothing filter array loader
  battRaw = (battSamp[0] + battSamp[1] + battSamp[2] + battSamp[3] + battSamp[4]) / 5;  // smoothing filter
  battLevel = battRaw * battCal;                                                        // uncomment for battery scaling method
  RemoteXY.batt_lvl = map(battLevel, 4, 6, 0, 100);  //Serial.print(i); Serial.println(battLevel); Serial.println(map(battLevel, 4, 6, 0, 100));

  // bug jitter - introduce organic movement sub here
  //j_mag = random(15); j_mag = j_mag - 5; strd_height = strd_height + j_mag;
  
  // reset movement to prepare for next move
  moveStand();
}

void moveStand()
{
  HCPCA9685.Servo(FLH, 210);HCPCA9685.Servo(FLK, constrain((225 - strd_height), 120, 360));         // move front left leg
  HCPCA9685.Servo(RRH, 200); HCPCA9685.Servo(RRK, constrain((225 - strd_height), 100, 340));        // move rear right leg
  HCPCA9685.Servo(FRH, 200); HCPCA9685.Servo(FRK, constrain((225 + strd_height), 100, 300));        // move front right leg
  HCPCA9685.Servo(RLH, 210); HCPCA9685.Servo(RLK, constrain((225 + strd_height), 120, 350));        // move rear left leg
}

void moveRotRight()
{
  HCPCA9685.Servo(FLK, constrain((360 + strd_height), 120, 360)); delay(move_s); HCPCA9685.Servo(FLH, 330); delay(move_s); HCPCA9685.Servo(FLK, constrain((225 - strd_height), 120, 360)); delay(move_s);
  HCPCA9685.Servo(RRK, constrain((340 - strd_height), 100, 340)); delay(move_s); HCPCA9685.Servo(RRH, 280); delay(move_s); HCPCA9685.Servo(RRK, constrain((210 - strd_height), 100, 340)); delay(move_s);
  HCPCA9685.Servo(FRK, constrain((100 + strd_height), 100, 300)); delay(move_s); HCPCA9685.Servo(FRH, 310); delay(move_s); HCPCA9685.Servo(FRK, constrain((225 + strd_height), 100, 300)); delay(move_s);
  HCPCA9685.Servo(RLK, constrain((120 - strd_height), 120, 350)); delay(move_s); HCPCA9685.Servo(RLH, 350); delay(move_s); HCPCA9685.Servo(RLK, constrain((350 - strd_height), 120, 350)); delay(move_s);
  moveStand();
}

void moveRotLeft()
{
  HCPCA9685.Servo(FLK, constrain((360 + strd_height), 120, 360)); delay(move_s); HCPCA9685.Servo(FLH, 130); delay(move_s); HCPCA9685.Servo(FLK, constrain((225 - strd_height), 120, 360)); delay(move_s);
  HCPCA9685.Servo(RRK, constrain((340 - strd_height), 100, 340)); delay(move_s); HCPCA9685.Servo(RRH, 100); delay(move_s); HCPCA9685.Servo(RRK, constrain((210 - strd_height), 100, 340)); delay(move_s);
  HCPCA9685.Servo(FRK, constrain((100 + strd_height), 100, 300)); delay(move_s); HCPCA9685.Servo(FRH, 100); delay(move_s); HCPCA9685.Servo(FRK, constrain((225 + strd_height), 100, 300)); delay(move_s);
  HCPCA9685.Servo(RLK, constrain((120 - strd_height), 120, 350)); delay(move_s); HCPCA9685.Servo(RLH, 150); delay(move_s); HCPCA9685.Servo(RLK, constrain((350 - strd_height), 120, 350)); delay(move_s);  
  moveStand();
}

void moveForward()
{
  HCPCA9685.Servo(FLK, constrain((360 + strd_height), 120, 360)); delay(move_s); HCPCA9685.Servo(FLH, 330); delay(move_s); HCPCA9685.Servo(FLK, constrain((225 - strd_height), 120, 360)); delay(move_s);
  HCPCA9685.Servo(RRK, constrain((340 - strd_height), 100, 340)); delay(move_s); HCPCA9685.Servo(RRH, 100); delay(move_s); HCPCA9685.Servo(RRK, constrain((210 - strd_height), 100, 340)); delay(move_s);
  HCPCA9685.Servo(FRK, constrain((100 + strd_height), 100, 300)); delay(move_s); HCPCA9685.Servo(FRH, 100); delay(move_s); HCPCA9685.Servo(FRK, constrain((225 + strd_height), 100, 300)); delay(move_s);
  HCPCA9685.Servo(RLK, constrain((120 - strd_height), 120, 350)); delay(move_s); HCPCA9685.Servo(RLH, 350); delay(move_s); HCPCA9685.Servo(RLK, constrain((350 - strd_height), 120, 350)); delay(move_s);
  moveStand();
}

void moveBackward()
{
  HCPCA9685.Servo(FLK, constrain((360 + strd_height), 120, 360)); delay(move_s); HCPCA9685.Servo(FLH, 130); delay(move_s); HCPCA9685.Servo(FLK, constrain((225 - strd_height), 120, 360)); delay(move_s);
  HCPCA9685.Servo(RRK, constrain((340 - strd_height), 100, 340)); delay(move_s); HCPCA9685.Servo(RRH, 280); delay(move_s); HCPCA9685.Servo(RRK, constrain((210 - strd_height), 100, 340)); delay(move_s);
  HCPCA9685.Servo(FRK, constrain((100 + strd_height), 100, 300)); delay(move_s); HCPCA9685.Servo(FRH, 310); delay(move_s); HCPCA9685.Servo(FRK, constrain((225 + strd_height), 100, 300)); delay(move_s);
  HCPCA9685.Servo(RLK, constrain((120 - strd_height), 120, 350)); delay(move_s); HCPCA9685.Servo(RLH, 150); delay(move_s); HCPCA9685.Servo(RLK, constrain((350 - strd_height), 120, 350)); delay(move_s);  
  moveStand();
}

void moveLeft()
{}

void moveRight()
{}

